@@ -198,11 +198,15 @@ command_parse_args ()
 	options)
 		_command_parse_args__chk_no_arg $#
 		;;
-	reconf|forget)
+	reconf|forget|escape|restore)
 		misc_chk_privilege
 		temp_warn_obsolete_temp_db
 		_command_parse_args__chk_glob_args $#
 		;;
+	pkgsanity)
+		misc_chk_privilege
+		temp_warn_obsolete_temp_db
+		;;
 	show)
 		database_query_chk_preparation_completion
 		temp_warn_obsolete_temp_db
@@ -238,7 +242,7 @@ command_parse_args ()
 		todo|done|redo|resolved|failure|taboo|need|noneed|restored|deleted|conflict)
 			_command_parse_args__chk_no_arg $#
 			;;
-		initrequirements|requirements|initdependents|dependents)
+		initrequirements|requirements|initdependents|dependents|status)
 			_command_parse_args__chk_glob_args $#
 			;;
 		*)
@@ -436,7 +440,7 @@ command_forget ()
 # ============= Execute command operations which are irrespective of option settings =============
 command_exec_irrespective_of_saved_options ()
 {
-	local dbfile tmp_manually_done_diff evalated_globs dbdir_parent dbdir_node arcfile grandtitle title isfirst origin origin_regexp
+	local dbfile tmp_manually_done_diff evalated_globs dbdir_parent dbdir_node arcfile grandtitle title isfirst origin origin_regexp backup_pkg origin_orig origin_replace pkg_orig tmp_done_orig tmp_pkgsanity nlines iline pkg
 	case $COMMAND_MODE in
 	ok)
 		dbfile=${DBDIR}/manually_done.list
@@ -504,6 +508,10 @@ command_exec_irrespective_of_saved_options ()
 		esac
 		str_escape_regexp_filter < ${DBDIR}/need.list \
 			| sed 's/^/^/; s/$/$/' > ${DBDIR}/need.grep_pattern
+		{
+			sed -E -f "${DBDIR}/REPLACE.complete_sed_pattern" "${DBDIR}/need.list" || :
+			cat "${DBDIR}/need.list" || :
+		} 2> /dev/null | sort -u > ${DBDIR}/need.with_replaced.list
 		program_deregister_stage_complete DETERMINE_SPECIFIED_TARGETS
 		_command_exec_irrespective_of_saved_options__notify_ignored_options
 		message_echo "Now the following ports are registered to be necessary:"
@@ -581,10 +589,10 @@ command_exec_irrespective_of_saved_options ()
 		message_echo "Temporary database sections for the following ports are reset for later rebuild:"
 		for origin in `pkgsys_eval_ports_glob "$@"`
 		do
-			database_build_make "$origin" config
 			origin_regexp=`str_escape_regexp "$origin"`
-			if grep -m 1 -E "^$origin_regexp$" "${DBDIR}/inspected_ports" > /dev/null 2>&1
+			if grep -q -E "^$origin_regexp$" "${DBDIR}/inspected_ports" 2> /dev/null
 			then
+				database_build_make "$origin" config
 				database_build_patch_reconf "$origin"
 			else
 				message_echo "$origin (not inspected)"
@@ -600,6 +608,164 @@ command_exec_irrespective_of_saved_options ()
 		command_forget "$@"
 		exit
 		;;
+	escape)
+		_command_exec_irrespective_of_saved_options__notify_ignored_options
+		message_echo "Backing up and deleting the following packages for a temporary escape:"
+		message_echo
+		for origin in `pkgsys_eval_ports_glob "$@"`
+		do
+			pkgsys_register_evaluated_globs add "${DBDIR}/taboo.list" "$origin"
+			message_echo "  Registered $origin as taboo."
+			pkg=`pkg_info_qO "$origin"` || continue
+			[ -n "$pkg" ] || continue
+			if backup_pkg=`pkgsys_get_backup_pkg "$origin"`
+			then
+				message_echo "INFO: A backup package for $pkg ($origin) already exists as $backup_pkg."
+			elif backup_pkg=`pkgsys_create_backup_pkg "$pkg" "${DBDIR}/backup_packages"`
+			then
+				message_echo "  Backed up $pkg ($origin) into $backup_pkg"
+			else
+				message_echo "ERROR: Failed to back up $pkg ($origin)." >&2
+				message_echo >&2
+				continue
+			fi
+			pkg_delete_f "$pkg" || \
+			{
+				message_echo "ERROR: Failed to deinstall $pkg ($origin)." >&2
+				message_echo >&2
+			}
+			message_echo "  Deinstalled $pkg ($origin)."
+			message_echo
+		done
+		fileedit_combine_lists "${DBDIR}/conf/TABOO:PORTS.parsed" "${DBDIR}/taboo.list" > ${DBDIR}/taboo.all.list
+		message_echo "Done"
+		exit
+		;;
+	restore)
+		_command_exec_irrespective_of_saved_options__notify_ignored_options
+		message_echo "Restoring the following temporary escaped packages:"
+		message_echo
+		tmp_done_orig=${TMPDIR}/command_exec_irrespective_of_saved_options::restore::done_orig
+		cp /dev/null "$tmp_done_orig"
+		for origin in `pkgsys_eval_ports_glob "$@"`
+		do
+			pkgsys_register_evaluated_globs remove "${DBDIR}/taboo.list" "$origin"
+			message_echo "  Deregistered $origin from taboo."
+			origin_regexp=`str_escape_regexp "$origin"`
+			grep -E -q "^$origin_regexp$" "$tmp_done_orig" || :
+			if pkg_info_eO "$origin"
+			then
+				pkg=`pkg_info_qO "$origin"` || :
+				message_echo "WARNING: $pkg ($origin) is already installed." >&2
+				message_echo >&2
+				continue
+			fi
+			origin_orig=`echo "$origin" \
+				| sed -E -f "${DBDIR}/REVERSE_REPLACE.complete_sed_pattern"`
+			if [ "x$origin_orig" = "x$origin" ] && pkg_info_eO "$origin_orig"
+			then
+				pkg_orig=`pkg_info_qO "$origin_orig"` || :
+				message_echo "WARNING: An original version of $origin ($pkg_orig, $origin_orig) is already installed." >&2
+				message_echo >&2
+				continue
+			fi
+			origin_replace=`echo "$origin" \
+				| sed -E -f "${DBDIR}/REPLACE.complete_sed_pattern"`
+			if [ "x$origin_replace" != "x$origin" ]
+			then
+				if pkg_info_eO "$origin_replace"
+				then
+					pkg_replace=`pkg_info_qO "$origin_replace"` || :
+					message_echo "WARNING: A replacement of $origin ($pkg_replace, $origin_replace) is already installed." >&2
+					message_echo >&2
+					continue
+				fi
+				if backup_pkg=`pkgsys_get_backup_pkg "$origin_replace" 2> /dev/null`
+				then
+					message_echo "INFO: $origin is replaced with $origin_replace ($pkg_replace)."
+					echo "$origin_replace" >> $tmp_done_orig
+					origin=$origin_replace
+				fi
+			else
+				backup_pkg=
+			fi
+			if [ -z "$backup_pkg" ] && ! backup_pkg=`pkgsys_get_backup_pkg "$origin" 2> /dev/null`
+			then
+				message_echo "ERROR: Backup for $origin is not found." >&2
+				message_echo >&2
+				continue
+			fi
+			pkg=`pkgsys_pkgarc_to_pkgname "$backup_pkg"`
+			if reinstall_chk_forbidden_conflicts "$pkg"
+			then
+				message_echo "WARNING: $pkg ($origin) is skipped because it conflicts with installed packages." >&2
+				message_echo >&2
+				continue
+			fi
+			if ! pkg_add_fF "$backup_pkg"
+			then
+				message_echo "ERROR: Failed to restore $pkg ($origin)." >&2
+				message_echo >&2
+			fi
+			message_echo "  Restored $pkg ($origin)."
+			message_echo
+		done
+		fileedit_combine_lists "${DBDIR}/conf/TABOO:PORTS.parsed" "${DBDIR}/taboo.list" > ${DBDIR}/taboo.all.list
+		message_echo "Done"
+		exit
+		;;
+	pkgsanity)
+		tmp_pkgsanity=${TMPDIR}/command_exec_irrespective_of_saved_options:pkgsanity
+		_command_exec_irrespective_of_saved_options__notify_ignored_options
+		if [ $# -eq 0 ]
+		then
+			message_echo "Sanity check of the installed files for each package:"
+			message_echo
+			pkg_info_Ea > $tmp_pkgsanity
+		else
+			message_echo "Examining the installed files for each specified package:"
+			message_echo
+			pkgsys_eval_ports_glob "$@" | while read origin
+			do
+				pkg_info_qO "$origin" || :
+			done | grep -v '^[[:space:]]*$' > $tmp_pkgsanity
+		fi
+		nlines=`wc -l < $tmp_pkgsanity`
+		iline=1
+		while [ $iline -le $nlines ]
+		do
+			pkg=`sed -n ${iline}p "$tmp_pkgsanity"`
+			iline=$((${iline}+1))
+			origin=`pkg_info_qo "$pkg"`
+			[ -n "$origin" ] || continue
+			origin_regexp=`str_escape_regexp "$origin"`
+			grep -q -E "^$origin_regexp$" "${DBDIR}/damaged_package" 2>/dev/null && continue
+			pkgsys_sanitychk_pkgcontents "$pkg" is_reinstall_encouraged && continue
+			if [ ! -d "${PORTSDIR}/$origin" ]
+			then
+				message_echo "WARNING: $pkg ($origin) is obsolete." >&2
+				message_echo >&2
+				continue
+			fi
+			if [ $is_reinstall_encouraged = no ]
+			then
+				if [ $opt_batch_mode = no ]
+				then
+					message_echo "Do you want to reinstall it? (y/[n])"
+					message_query_yn_default_no || continue
+				fi
+			else
+				if [ $opt_batch_mode = no ]
+				then
+					message_echo "Do you want to reinstall it? ([y]/n)"
+					message_query_yn_default_yes || continue
+				fi
+				database_record_reconf_recover_sanity "$origin"
+			fi
+		done
+		message_echo "Done"
+		exit
+		;;
 	redo)
 		if [ $opt_reload_conf = yes ]
 		then
@@ -738,6 +904,12 @@ command_exec_before_tools_upgrade ()
 			database_query_for_each_matching_port "$grandtitle" "$title" "$list" "$pkgnamedb" \
 				"$COMMAND_SHOW_DEPTAG" "$COMMAND_SHOW_LEVEL" "$@"
 			;;
+		status)
+			grandtitle="Success/failure status in (re)installation"
+			lists='todo|done|redo|resolved|failure|taboo|need|noneed|deleted|restored|conflict'
+			database_query_for_list_inclusion_of_matching_port "$grandtitle" "$lists" "$pkgnamedb" \
+				"$COMMAND_SHOW_DEPTAG" "$COMMAND_SHOW_LEVEL" "$@"
+			;;
 		esac
 		exit
 		;;
