@@ -864,20 +864,28 @@ _database_build_reset_a_port_confdb ()
 	cat "${DBDIR}/replace/$origin/origin" 2> /dev/null || :
 }
 
-# ============= Remove configurations for a port permanently =============
-database_build_forget ()
+# ============= Clear database directories for an origin =============
+database_build_clear_db_dirs ()
 {
-	local origin origin_replace db
+	local origin db
 	origin=$1
-	origin_replace=`_database_build_reset_a_port_confdb "$origin"`
-	fileedit_rm_a_line "$origin" "${DBDIR}/targets_specified_so_far"
-	[ -z "$origin_replace" ] || database_build_forget "$origin_replace"
 	for db in requires replace targets obsolete
 	do
 		rm -rf "${DBDIR}/$db/$origin"
 	done
 }
 
+# ============= Remove configurations for a port permanently =============
+database_build_forget ()
+{
+	local origin origin_replace
+	origin=$1
+	origin_replace=`_database_build_reset_a_port_confdb "$origin"`
+	fileedit_rm_a_line "$origin" "${DBDIR}/targets_specified_so_far"
+	[ -z "$origin_replace" ] || database_build_forget "$origin_replace"
+	database_build_clear_db_dirs "$origin"
+}
+
 # ============= Patch to the temporary database so as to re-inspect and reinstall ports whose configurations were changed =============
 database_build_patch_reconf ()
 {
@@ -888,13 +896,10 @@ database_build_patch_reconf ()
 		fileedit_add_a_line_if_new "$origin" "${DBDIR}/stage.loop_list/ports_to_inspect"
 	fileedit_add_a_line_if_new "$origin" "${DBDIR}/to_be_reconf"
 	[ -z "$origin_replace" ] || database_build_patch_reconf "$origin_replace"
-	for db in requires replace targets obsolete
-	do
-		rm -rf "${DBDIR}/$db/$origin"
-	done
+	database_build_clear_db_dirs "$origin"
 }
 
-# ============= Post`processes after finishing to inspect dependencies =============
+# ============= Post-processes after finishing to inspect dependencies =============
 database_build_post_inspect_dependencies ()
 {
 	local table
