@@ -20,23 +20,29 @@ database_query_show_list_failure ()
 		origin_regexp=`str_escape_regexp "$origin"`
 		grep -q -E "^$origin_regexp$" "${DBDIR}/stage.loop_list/ports_to_delete" 2> /dev/null \
 			&& continue
-		note=`cat "${DBDIR}/requires/$origin/note_failtre"`
+		note=`cat "${DBDIR}/notes/$origin/note_failtre" 2> /dev/null || :`
 		resolved=no
 		grep -q -E "^`str_escape_regexp \"$origin\"`$" \
 			"${DBDIR}/manually_done.list" 2> /dev/null \
 			&& resolved=yes
-		pkgtag=`cat "${DBDIR}/requires/$origin/pkgtag" 2> /dev/null` || :
+		pkgtag=`cat "${DBDIR}/requires/$origin/pkgtag" 2> /dev/null || :`
 		if [ $opt_batch_mode = no ]
 		then
 			case $resolved in
 			no)	resolved=;;
 			yes)	resolved=', resolved';;
 			esac
+			if [ -n "$note" ]
+			then
+				detail="while [$note]"
+			else
+				detail="by unrecorded reasons"
+			fi
 			if [ -n "$pkgtag" ]
 			then
-				echo "$origin ($pkgtag) (error while [$note]$resolved)"
+				echo "$origin ($pkgtag) (error $detail$resolved)"
 			else
-				echo "$origin (error while [$note]$resolved)"
+				echo "$origin (error $detail$resolved)"
 			fi
 		else
 			printf "%s\t%s\t%s\t%s\n" "$origin" "$pkgtag" "$note" "$resolved"
