@@ -43,7 +43,7 @@ database_record_success ()
 	fileedit_rm_a_line "$origin" "${DBDIR}/failed.list"
 	fileedit_rm_a_line "$origin" "${DBDIR}/damaged_package"
 	touch "$nodedir/succeeded_once"
-	rm -f "$nodedir/note_failtre" "$nodedir/conf_updated"
+	rm -f "${DBDIR}/notes/$origin/note_failtre" "$nodedir/conf_updated"
 }
 
 # ============= Update records by reflecting a failed result of (re/de)installation of a port =============
