@@ -51,14 +51,14 @@ message_query_yn_default_no ()
 # ============= Output the version =============
 message_version ()
 {
-	[ $opt_batch_mode = no ] && echo -n "${APPNAME} version "
+	[ "x$opt_batch_mode" = xno ] && echo -n "${APPNAME} version "
 	echo "${MYVERSION}"
 }
 
 # ============= Output a message except for the batch mode =============
 message_echo ()
 {
-	[ $opt_batch_mode = yes ] && return
+	[ "x$opt_batch_mode" = xyes ] && return
 	echo "$@" | fold -s -w `misc_get_console_column_size`
 }
 
@@ -67,7 +67,7 @@ message_echo ()
 message_cat ()
 {
 	local ncolumns
-	[ $opt_batch_mode = yes ] && return
+	[ "x$opt_batch_mode" = xyes ] && return
 	ncolumns=`misc_get_console_column_size`
 	if [ $# -eq 0 ]
 	then
@@ -81,7 +81,7 @@ message_cat ()
 message_credit ()
 {
 	local ncolumns
-	[ $opt_batch_mode = yes ] && return
+	[ "x$opt_batch_mode" = xyes ] && return
 	ncolumns=`misc_get_console_column_size`
 	message_version | fold -s -w $ncolumns
 	fold -s -w $ncolumns << eof
@@ -103,7 +103,7 @@ message_timestamp ()
 message_opening_notice ()
 {
 	local ncolumns
-	[ $opt_batch_mode = yes ] && return
+	[ "x$opt_batch_mode" = xyes ] && return
 	ncolumns=`misc_get_console_column_size`
 	fold -s -w $ncolumns << eof
 
@@ -125,7 +125,7 @@ message_report_failure_for_a_port ()
 # ============= Output the relation of the current port with specified targets =============
 message_target_relations ()
 {
-	[ $opt_batch_mode = yes ] && return
+	[ "x$opt_batch_mode" = xyes ] && return
 	( set -e
 		origin=$1
 		database_query_get_target_attributes it "$origin"
