@@ -8,6 +8,7 @@
 
 # ============= Variables =============
 MISC_IS_READONLY_MODE=no
+MISC_DEFAULT_CONSOLE_SIZE='25 80'
 
 # ============= Check whether the current process is by the superuser privilege =============
 misc_is_superuser_privilege ()
@@ -73,6 +74,7 @@ misc_get_console_size ()
 			size=`cat "${TMPDIR}/misc_get_console_size::size"`
 		fi
 	fi
+	[ -n "$size" ] || size=$MISC_DEFAULT_CONSOLE_SIZE
 	echo "$size"
 }
 
