@@ -44,7 +44,7 @@ d	keep-distfiles	opt_keep_distfiles	no	yes
 k	suppress-self-upadte	opt_suppress_self_upadte	no	yes
 g	suppress-pkgtools-upadte	opt_suppress_pkgtools_upadte	no	yes
 c	suppress-clean-obsolete-database	opt_suppress_obsolete_db_clean	no	yes
-D	suppress-entire-inspection-distinfo	opt_suppress_inspect_entire_distinfo	yes	no
+D	suppress-entire-inspection-distinfo	opt_inspect_entire_distinfo	yes	no
 n	dry-run	opt_dry_run	no	yes
 X	deselect-all	opt_dialog_auto	no	none
 Y	select-all	opt_dialog_auto	no	all
