@@ -7,11 +7,11 @@
 # ==============================================================================
 
 # ============= Variables =============
-PKGSYS_USE_PKGNG=no	# no: legacy pkg_* tools, yes: the new generation package (pkgng)
-PKGSYS_CMD_PKG_INFO='pkg_info'	# Corresponding command for pkg_info
-PKGSYS_CMD_PKG_CREATE='pkg_create'	# Corresponding command for pkg_create
-PKGSYS_CMD_PKG_DELETE='pkg_delete'	# Corresponding command for pkg_delete
-PKGSYS_CMD_PKG_ADD='pkg_add'	# Corresponding command for pkg_add
+PKGSYS_USE_PKGNG=yes	# no: legacy pkg_* tools, yes: the new generation package (pkgng)
+PKGSYS_CMD_PKG_INFO='pkg info'	# Corresponding command for pkg_info
+PKGSYS_CMD_PKG_CREATE='pkg create'	# Corresponding command for pkg_create
+PKGSYS_CMD_PKG_DELETE='pkg delete'	# Corresponding command for pkg_delete
+PKGSYS_CMD_PKG_ADD='pkg add'	# Corresponding command for pkg_add
 PKGSYS_AVR_REFETCH_TIMES_PER_SITE=1	# Average number (integer) of retrials for retrieving package or distfiles per mirror site
 PKGSYS_AVR_REFETCH_TIMES_FOR_CHKSUMERR=2	#  Number (integer) of retrials for check sum error in retrieving a package
 
@@ -23,7 +23,7 @@ pkgsys_sysvar ()
 	tmp_work=${TMPDIR}/pkgsys_sysvar:work
 	rm -rf "$tmp_work"
 	mkdir "$tmp_work"
-	make -C "$tmp_work" -f "${PORTSDIR}/Mk/bsd.port.mk" -V "$var"
+	make -C "$tmp_work" -f "${PORTSDIR}/Mk/bsd.port.mk" -V "$var" 2> /dev/null
 }
 
 # ============= Get the file name of package check sum file =============
@@ -214,10 +214,16 @@ pkg_is_dependency_of_a_legacypkg_latest ()
 	:
 }
 
+# ============= Check whether legacy package tools are available =============
+pkgsys_is_legacy_tool_available ()
+{
+	which -s pkg_info
+}
+
 # ============= Define wrapper commands for hiding the differences between pkg_* tools and pkgng =============
 pkgsys_def_pkgtools ()
 {
-	if [ "${DBDIR}/WITH_PKGNG" -nt /etc/make.conf ]
+	if [ "${DBDIR}/WITH_PKGNG" -nt /etc/make.conf -o \( -e "${DBDIR}/WITH_PKGNG" -a ! -e /etc/make.conf \) ]
 	then
 		PKGSYS_USE_PKGNG=`cat "${DBDIR}/WITH_PKGNG"`
 	else
@@ -237,7 +243,14 @@ pkgsys_def_pkgtools ()
 		PKGSYS_CMD_PKG_ADD='pkg add'
 		pkg_is_tool_available ()
 		{
-			which -a pkg | grep -v '^/usr/sbin/pkg$' | grep -q '/sbin/pkg$'
+			if [ -x /usr/sbin/pkg ]
+			then
+				pkg -N 2> /dev/null && return
+				env ASSUME_ALWAYS_YES=yes pkg bootstrap -f
+				pkg_is_tool_available
+			else
+				which -s pkg && return
+			fi
 		}
 		pkg_info_Ea ()
 		{
@@ -399,6 +412,7 @@ pkgsys_def_pkgtools ()
 			then
 				pkg_inst_remote "$pkg" "$mode" && return
 			fi
+			pkgsys_is_legacy_tool_available || return
 			message_echo "INFO: Trying to use a legacy package and convert it to pkgng."
 			pkgsys_fetch_legacy_remote "$pkg" || return
 			pkgarc=`pkgsys_fetch_legacy_remote__fetched_pkg`
@@ -450,23 +464,42 @@ pkgsys_def_pkgtools ()
 		}
 		pkg_rescue_tools ()
 		{
-			local packagepath checksumpath pkgname
+			local packagepath checksumpath pkgname is_successful
 			packagepath=`pkgsys_get_backup_pkg ports-mgmt/pkg` && \
 				pkg_add_tools "$packagepath" && return
-			[ -x /usr/local/sbin/pkg ] && return
-			message_echo "WARNING: WITH_PKGNG is set, but pkgng is missing. It is installed now." >&2
-			[ -x /usr/sbin/pkg ] && /usr/sbin/pkg && return
+			pkg_is_tool_available && return
+			message_echo "WARNING: WITH_PKGNG is set, but pkgng is still missing. It is installed now." >&2
 			pkgsys_ready_checksum_file || return
+			message_echo "INFO: Installing pkgng by legacy package tool."
 			checksumpath=`pkgsys_ready_checksum_file__fetched_file`
 			pkgname=`sed 's/^MD5[[:space:]]*(//;s/\.tbz)[[:space:]]*=[^=]*$//' "$checksumpath" \
 				| grep -m 1 -E -e "^pkg-[0-9]"` || :
-			[ -z "$pkgname" ] && pkg_inst_remote_wild "$pkgname" nodepschk && return
-			make -C "${PORTSDIR}/ports-mgmt/pkg" install clean && return
-			message_echo "ERROR: WITH_PKGNG is set, but pkgng is missing. It is installed now." >&2
-			:
+			[ -n "$pkgname" ] && pkg_inst_remote_wild "$pkgname" nodepschk && return
+			message_echo "INFO: Failed by package, so installing pkgng by port."
+			grep -v '^[[:space:]]*WITH_PKGNG=' /etc/make.conf > ${TMPDIR}/make.conf
+			echo WITHOUT_PKGNG=yes >> ${TMPDIR}/make.conf
+			( set -e
+				unset WITH_PKGNG
+				unset WITHOUT_PKGNG
+				
+				message_echo "INFO: Attempting deinstallation of ports-mgmt/pkg to make sure."
+				env __MAKE_CONF="${TMPDIR}/make.conf" make -C "${PORTSDIR}/ports-mgmt/pkg" deinstall || :
+				message_echo "INFO: Attempting (re)installation by ports-mgmt/pkg."
+				env __MAKE_CONF="${TMPDIR}/make.conf" make -C "${PORTSDIR}/ports-mgmt/pkg" reinstall clean
+			) && {
+				pkg2ng || :
+				pkg_is_tool_available
+			}
 		}
-		pkg_rescue_tools
+		if ! pkg_rescue_tools
+		then
+			message_echo "WARNING: Pkgng is still missing, but continuing for the time being." >&2
+		fi
 		pkg_loadconf
+	elif ! pkgsys_is_legacy_tool_available
+	then
+		message_echo "ERROR: Pkgng is disabled although the legacy packages tools are unavailable. Resolve the problem manually." >&2
+		exit 1
 	else
 		unset WITH_PKGNG
 		PKGSYS_USE_PKGNG=no
@@ -476,7 +509,7 @@ pkgsys_def_pkgtools ()
 		PKGSYS_CMD_PKG_ADD='pkg_add'
 		pkg_is_tool_available ()
 		{
-			:
+			pkgsys_is_legacy_tool_available
 		}
 		pkg_info_Ea ()
 		{
@@ -650,17 +683,21 @@ pkgsys_init_pkg_orig_by_ambiguous_matching ()
 	ambsuffix=
 	norigins=0
 	len_pkg=`echo -n "$pkg" | wc -c`
-	while [ $len_pkg -gt 0 ]
-	do
-		pkg_regexp=`str_escape_regexp "$pkg"`$ambsuffix
-		grep -E "^${pkg_regexp}[[:space:]]" "${DBDIR}/installed_ports:pkg_vs_origin.tbl" 2> /dev/null \
-			| cut -f 2 > $tmporigin
-		norigins=`wc -l < $tmporigin`
-		[ $norigins -gt 0 ] && break
-		len_pkg=$(($len_pkg-1))
-		pkg=`echo -n "$pkg" | head -c $len_pkg`
-		ambsuffix='[a-zA-Z0-9.,_+-]*'
-	done
+	if [ $len_pkg -gt 0 ]
+	then
+		while :
+		do
+			pkg_regexp=`str_escape_regexp "$pkg"`$ambsuffix
+			grep -E "^${pkg_regexp}[[:space:]]" "${DBDIR}/installed_ports:pkg_vs_origin.tbl" 2> /dev/null \
+				| cut -f 2 > $tmporigin
+			norigins=`wc -l < $tmporigin`
+			[ $norigins -gt 0 ] && break
+			ambsuffix='[a-zA-Z0-9.,_+-]*'
+			len_pkg=$(($len_pkg-1))
+			[ $len_pkg -gt 0 ] || break
+			pkg=`echo -n "$pkg" | head -c $len_pkg`
+		done
+	fi
 	[ $norigins -eq 1 ] || return
 	cat "$tmporigin"
 }
