@@ -131,7 +131,8 @@ reinstall_errproc ()
 	message_echo "The port/package in concern is $REINSTALL_ORIGPKGTAG." >&2
 	[ -n "$msg" ] && message_echo "($msg)" >&2
 	message_echo >&2
-	echo "$position" > $REINSTALL_DBNODE_DIR/note_failtre
+	[ -d "${DBDIR}/notes/$REINSTALL_ORIGIN" ] || mkdir -p "${DBDIR}/notes/$REINSTALL_ORIGIN"
+	echo "$position" > ${DBDIR}/notes/$REINSTALL_ORIGIN/note_failtre
 	database_record_failure "$REINSTALL_ORIGIN" noclean
 	fileedit_rm_a_line "$REINSTALL_ORIGIN" "${DBDIR}/stage.loop_list/reinst_todo.remain"
 	message_report_failure_for_a_port "$REINSTALL_ORIGIN"
@@ -696,26 +697,44 @@ reinstall_exec ()
 		fi
 		if reinstall_chk_stage FAILED_FETCH
 		then
-			message_echo "INFO: Refetching distfiles for $REINSTALL_ORIGPKGTAG."
-			if ! reinstall_chk_stage_complete FAILED_FETCH.REFETCH
+			if ! reinstall_chk_stage_complete FAILED_FETCH.RETRIAL_1
 			then
-				if ! reinstall_make_anymode fetch FETCH_ARGS=-Ap
-				then
-					reinstall_restore_conflicts "$REINSTALL_ORIGIN"
-					reinstall_restore_if_temporarily_deinstalled
-					reinstall_errproc 'fetch'
-					temp_set_msg_current_stage "${_MSG_CURRENT_STAGE_general}"
-					return
-				fi
-				reinstall_make_anymode checksum || \
+				message_echo "INFO: Refetching distfiles for $REINSTALL_ORIGPKGTAG."
 				{
-					reinstall_restore_conflicts "$REINSTALL_ORIGIN"
-					reinstall_restore_if_temporarily_deinstalled
-					reinstall_errproc 'checksum'
-					temp_set_msg_current_stage "${_MSG_CURRENT_STAGE_general}"
-					return
-				}
-				reinstall_register_stage_complete FAILED_FETCH.REFETCH
+					reinstall_make_anymode fetch FETCH_ARGS=-Ap &&
+						reinstall_make_anymode checksum
+				} || reinstall_register_stage FAILED_REFETCH_1
+				reinstall_register_stage_complete FAILED_FETCH.RETRIAL_1
+			fi
+			if reinstall_chk_stage FAILED_REFETCH_1
+			then
+				if ! reinstall_chk_stage_complete FAILED_FETCH.RETRIAL_2.DISTCLEAN
+				then
+					message_echo "INFO: Cleaning distfiles for the second refetch for $REINSTALL_ORIGPKGTAG."
+					reinstall_make_anymode distclean || :
+					reinstall_register_stage_complete FAILED_FETCH.RETRIAL_2.DISTCLEAN
+				fi
+				if ! reinstall_chk_stage_complete FAILED_FETCH.RETRIAL_2
+				then
+					message_echo "INFO: Refetching distfiles as the second retrial for $REINSTALL_ORIGPKGTAG."
+					if ! reinstall_make_anymode fetch FETCH_ARGS=-Ap
+					then
+						reinstall_restore_conflicts "$REINSTALL_ORIGIN"
+						reinstall_restore_if_temporarily_deinstalled
+						reinstall_errproc 'fetch'
+						temp_set_msg_current_stage "${_MSG_CURRENT_STAGE_general}"
+						return
+					fi
+					reinstall_make_anymode checksum || \
+					{
+						reinstall_restore_conflicts "$REINSTALL_ORIGIN"
+						reinstall_restore_if_temporarily_deinstalled
+						reinstall_errproc 'checksum'
+						temp_set_msg_current_stage "${_MSG_CURRENT_STAGE_general}"
+						return
+					}
+					reinstall_register_stage_complete FAILED_FETCH.RETRIAL_2
+				fi
 			fi
 		fi
 		reinstall_register_stage_complete FETCH
@@ -792,7 +811,7 @@ reinstall_exec ()
 				return
 			}
 			reinstall_register_stage_complete CLEAN_BEFORE_BUILD
-			reinstall_make build || \
+			reinstall_make build MAKE_JOBS_UNSAFE=yes || \
 			{
 				reinstall_restore_conflicts "$REINSTALL_ORIGIN"
 				reinstall_restore_if_temporarily_deinstalled
@@ -843,7 +862,7 @@ reinstall_exec ()
 					reinstall_make deinstall \
 						|| message_echo "WARNING: Continuing by hoping a success." >&2
 					message_echo "INFO: Retrying the installation."
-					reinstall_make $insttarget
+					reinstall_make $insttarget MAKE_JOBS_UNSAFE=yes
 				}
 			then
 				touch "$REINSTALL_DBNODE_DIR/installed_timestamp"
